***********************************************************

* Boot FLEX on a Dragon 64Kram and a DriveWire server
* 
* Version 0.1   06/2023 from Grard Huyghe /Azerpy 
* with the benefit of all before me (Tormod, Pere, Phil, 
* Sixxie, the DW conceptors, Stew and all I forget here ...

************************************************************ 
* Code to create a file.BIN to install at $4D00-$5259
* or a DWEEB (with header and queue) for DLOAD or DWLOAD 
* No DOS needed, uses the same (included) drivers to boot and * to run FLEX, for emulation or real Dragon (auto detected)
* Use with one or two Flex disks installed in a DW4 or a
* Py DriveWire server, with a Dragon bootable flex disk
* in Drive 0 and any other flex disk in Drive 1
*            
***********************************************************
 
 ORG $4D00
 PUT *
		bra	START

***********************************************************
OUTNUM   equ   $957A       ;print number in D to DEVNUM
OUTCHR   equ   $B54A       ;print char A to DEVNUM
OUTSTR   equ   $90E5       ;print string at X+1 to DEVNUM
initmsg	fcb $0D,/BOOT FLEX WITH DW V0.1/,$0D,0
FLAG		fcb	0
* DATE buffer YY year minus 1900,MM month 1-12,DD day 1-31 
DATEBUFF	fcb	123,3,1,0,0,0

* fail access to DW during boot
failmsg 	fcb $0D,/ACCES TO DW ?.../,$0D,0
FAIL		ldx #failmsg -1
		jsr OUTSTR
		rts
* abort if disk in Drive 1 or Drive 0 seems odd
* test number of sect/track <10 or > 36
oddmsg	fcb $0D,/ODD DISK /,$0D,0
ODD		ldx #oddmsg -1
		jsr OUTSTR
		rts

***********************************************************
START		ldx 	#initmsg -1
		jsr OUTSTR

* try access to DriveWire by loading DW date
* first with HDBDOS drivers DWRead & DWWrite for real Dragon
START1	lda #$23
		pshs a
		leax  ,s
		ldy #1
		jsr DWWrite
		puls a
		ldx #DATEBUFF
		ldy #6
		jsr DWRead
		beq START2

* on error, try once with drivers for emulation / Becker Port
		lda	FLAG
		bne FAIL
		jsr CHANGE		; go sub change drivers
		inc FLAG
		bra START1

* Drivers OK, print date of today
START2	jsr	DATE

* load boot sector 1 of the flex disk in Drive 0
* and now test access to Drive 0
		ldd #$0001
		ldx #$2500
		jsr READFLX
		lbne FAIL		; Read error with Drive 0

* load boot sector 3 of the flex disk in Drive 1 
		lda # 1
		sta DNUM
		ldd #$0003
		ldx #$2600	; temporary storage
		jsr READFLX
		beq	START3
		clr	DSKMAX	; no disk in DRIVE 1
		bra	START4
START3	lda	$26,x		; get number of sectors & tracks
		sta	TRKNUM1
		lda	$27,x
		cmpa	#36
		lbgt	ODD
		cmpa	#10
		lblt	ODD		; abort if seems not a flex disk
		sta	SECNUM1

* load boot sector 3 of the flex disk in drive 0
START4	clr DNUM
		ldd #$0003
		ldx #$2600
		jsr READFLX
		lbne FAIL
		lda	$26,x		; get number of sectors & tracks
		sta	TRKNUM0
		sta	TRKNUM
		lda	$27,x
		cmpa	#36
		lbgt	ODD
		cmpa	#10
		lblt	ODD		; abort if seems not a flex disk
		sta	SECNUM0
		sta	SECNUM

* print disks infos and wait for go on 
		jsr	DISKINF1
inloop	jsr	$8009		; cursor blink
		jsr	$8006		; scan keyboard
		beq	inloop

* the flex boot is now installed from $2500 to $26d7
* patch it to use DW Drivers
* see disassemble of the boot for more
		lda #$7e 
		sta $26a2		
		ldx #$2500		
		stx $26a3		; JMP $2500

		lda #$7e 
		sta $25b6	
		ldx #READFLX
		stx $25b7		; JMP READFLX

		lda #$bd  
		sta $25ae 
		ldx #COPY
		stx $25af		; JSR COPY
		lda #$12 
		sta $25b1		; NOP
*		ldx #$BFFF
*		stx $269E		; end copy RAM, no DOS needed

* BOOT FLEX via $2602 then $2500 then GO FLEX $CD00
		jmp $2602

******************************************************
* subroutines
******************************************************
* if emulation detected, sub to replace code DWRead & DWWrite
CHANGE	nop
* if DOS present, change also $FF42 by $FF4A
		ldx	$C000
		cmpx	#$444B	;"DK" in $C000
		bne CHANGE1
		lda	#$4A
		sta	ADW+2
		sta	ADR+2
		deca
		sta	ADR-5

CHANGE1	ldx #DWReadBK
		ldy #DWRead
LOOP3		lda	,x+
		sta	,y+
		cmpx #DWWriteBK -1
		ble 	LOOP3	

		ldx #DWWriteBK
		ldy #DWWrite
LOOP4		lda	,x+
		sta	,y+
		cmpx #DWWriteBK +13
		ble 	LOOP4
		rts

******************************************************
* sub copy drivers to $DE00 for FLEX
* end of disk drivers  max 525d / e05d 
COPY		ldx #$5000
		ldy #$de00
LOOP5		lda	,x+
		sta	,y+
		cmpx #$525D
		ble 	LOOP5
		rts

*******************************************************
* sub print DATE
datemsg	fcb /TODAY IS /,0
DATE		ldx 	#datemsg -1
		jsr OUTSTR
		clra
         	ldb   DATEBUFF
         	addd  #1900
         	jsr   OUTNUM
         	lda   #'-
         	jsr   OUTCHR
         	ldb   DATEBUFF+1
         	bsr   print2d
        	lda   #'-
         	jsr   OUTCHR
         	ldb   DATEBUFF+2
         	bsr   print2d
         	lda   #13
         	jsr   OUTCHR
		rts
print2d  	cmpb  #9
         	bhi   twod
         	lda   #'0'
         	jsr   OUTCHR
twod     	clra
         	jmp   OUTNUM

*******************************************************
* sub print disks info
bootmsg0	fcb $0D,/DISK IN DRIVE 0/,$0D,0
bootmsg1	fcb $0D,/DISK IN DRIVE 1/,$0D,0
bootmsg2	fcb / SECTORS BY TRACK/,$0D,0
bootmsg3	fcb / TRACKS/,$0D,0
bootmsg4   fcb /don't change disks after boot /,0
bootmsg5   fcb /NO DISK IN DRIVE 1/,0

DISKINF1	lda	DSKMAX
		cmpa	#0
		beq	NODSK1	; no disk 1

* infos disk 1
		ldx 	#bootmsg1 -1
		jsr OUTSTR
		clra
		ldb	SECNUM1
		jsr	OUTNUM
		ldx 	#bootmsg2 -1
		jsr OUTSTR
		clra
		ldb	TRKNUM1
		incb
		jsr	OUTNUM
		ldx 	#bootmsg3 -1
		jsr OUTSTR

* infos disk 0
DISKINF2	ldx 	#bootmsg0 -1
		jsr OUTSTR
		clra
		ldb	SECNUM0
		jsr	OUTNUM
		ldx 	#bootmsg2 -1
		jsr OUTSTR
		clra
		ldb	TRKNUM0
		incb
		jsr	OUTNUM
		ldx 	#bootmsg3 -1
		jsr OUTSTR

* end of infos
		lda	#$0D		
		jsr OUTCHR  
 		ldx 	#bootmsg4 -1
		jsr OUTSTR
		rts

NODSK1	ldx 	#bootmsg5 -1
		jsr OUTSTR
		bra	DISKINF2

*******************************************************
* this code will replace DWRead & DWWrite in case of emulation
* with Becker port, from Pere's DOSPlusExt, uses $FF42 address

 ORG $4FA0
 PUT *

DWReadBK   clra      ; clear Carry (no framing error)
         	deca        ; clear Z flag, A = timeout msb ($ff)
         	tfr	cc,b
         	pshs  u,x,dp,b,a    ; preserve registers, push flags and timeout msb
         	leau  ,x		; U points to received data buffer
         	ldx   #$0000		; checksum to zero
         	leas	-2,s		; create a word hole for a 16 bits counter
loop0    	ldd	#$F000	; reset counter to a fixed value
		std	,s								; to count 16 times 256
loop1    	dec	1,s		; decrement low byte counter
		bne	loop2		; if not zero, skip next byte control
		inc	,s		; increment high byte
		beq	notReady	; if zero, exit via notReady
loop2		ldb   $FF41	; pol if DW is ready
         	bitb  #$02		; is ready?
         	beq   loop1	; no, loop
ADR		ldb   $FF42	; get data byte
         	stb   ,u+		; put at destination buffer
         	abx			; accumulate in checksum word
         	leay  ,-y		; decrement byte counter
	    	bne	loop0		; not all received, loopback
		leas	2,s		; discard word counter off the stack
         	tfr   x,y	; pass checksum to Y
         	ldb   #0
         	lda   #3
         	leas  1,s    	; remove timeout msb from stack
outLoop  	inca       	; A = status to be returned in C and Z (bit2-Z; bit0-C), now A=4 (Z=1 C=0)
         	ora   ,s     ; place status information into the..
         	sta   ,s       ; ..C and Z bits of the preserved CC
         	leay  ,x       ; get checksum into Y
         	puls  cc,dp,x,u,pc   ; restore registers and return

notReady 	leas	3,s	; to remove word counter and timeout
		clra		; to get a #$01 in next opcode (to set Carry flag)
		bra	outLoop	; update flags and exit

		rmb 3

DWWriteBK  pshs  d,cc        ; preserve registers
txByteBK	lda   ,x+
ADW        sta   $FF42
         	leay  -1,y        ; decrement byte counter
         	bne   txByteBK    ; loop if more to send
         	puls  cc,d,pc     ; restore registers and return

*******************************************************
*
*              DW DRIVERS FOR FLEX
*
 
* used at $5000 for boot 
* and copied at $DE00 for running FLEX 
*
* as described in the FLEX Advanced Programmer's Guide
*
*******************************************************

 ORG $5000
 PUT *
BBIN		EQU	$FF22		; bit banger input port
BBOUT		EQU	$FF20		; bit banger output port
IntMasks	EQU	$50		; set I,F
DELAY		EQU	$0085		; timeout for DWRead

*         TABLE OF DISK DRIVERS *
READ			JMP READFLX
WRITE			JMP WRITEFLX
VERIFY		JMP OK
RESTORE		JMP OK
DRIVESELECT	JMP DRIVE
CHKRDY		JMP CHCK
QUICK			JMP CHCK
COLDINIT		JMP INIT
WARMINIT		JMP OK
SEEK			JMP OK

* Drive number
DNUM		FCB	0
DSKMAX	FCB	1		; number of last disk (0 or 1)


* sectors per track of current disk/disk0/disk1
* used to calculate Logical Sector Number, 
* may be different of 18 sectors per track
SECNUM	FCB 18
SECNUM0 	FCB 18
SECNUM1	FCB 18

* last track of current disk/disk0/disk1
* only used for initial information, no control
TRKNUM	FCB 39
TRKNUM0	FCB 39
TRKNUM1	FCB 39

* Error condition code (used for control only)
ReadErr	FCB 0
WriteErr	FCB 0

* DRIVE SELECT
DRIVE		lda	3,x
		sta	DNUM,pcr
		cmpa #0
		bne	DR1
DR0		ldb	SECNUM0,pcr
		stb	SECNUM,pcr
		bra	OK
DR1		ldb	SECNUM1,pcr
		stb	SECNUM,pcr
		bra	OK

* CHECK DRIVE READY
CHCK		lda	3,x
		cmpa DSKMAX,pcr	
		bgt	NOTREADY
OK		clrb
		rts

* if timeout or DW error code $F6 during Read or Write sector
NOTREADY	ldb	#$80
		rts
* if other DW error code ($f3,f4,f5)
ERRCODE	cmpb	#$F6
		beq	NOTREADY
		ldb	#$08
		rts

* first initialisation of Flex after boot
* avoids date ask and sets date given by DW server
INIT		clrb
		stb DNUM,pcr
		lda DATEBUFF+1
		sta $CC0E
		lda DATEBUFF+2
		sta $CC0F
		lda DATEBUFF
		suba #100
		sta $CC10
		lda #$39	; poke an RTS 
		sta $CAAF	; at flex sub ask date 
		rts

****************************************************
* sub READ sector of 256 Bytes
* Input A=Track, B= Sector, X = Buffer Address
* Output X preserved

READFLX	pshs b,x

* Calculate X= LSN =SECNUM*T+S-1
		ldb	SECNUM,pcr
		mul
		tfr d,x
		puls b
		abx
		leax -1,x
		pshs	x	; stack = LSN/BufferAd

* stack bytes to send
		lda	DNUM,pcr
         	clrb
         	pshs  a,b
         	lda   #$D2
	   	pshs  a	; stack = $D2/DNUM/0/LSN/BufferAd
* send  5 bytes to DW         	
		leax  ,s    ; get start address
		ldy   #5
		lbsr  DWWrite
* read 256 bytes
		ldx   5,s	  ; get BufferAd
		ldy   #256			
		ldd   #DELAY		
		lbsr   DWRead
		lbne	NOTREADY
* Send 2 byte checksum given by DWRead
		pshs  y
		leax  ,s    ; get start address
		ldy   #2
		lbsr  DWWrite
* Get control code
		ldy   #1
		leax  ,s    ; get start address
		ldd   #DELAY
		lbsr   DWRead
		lbne	NOTREADY
		ldb   ,s
		stb	ReadErr,pcr
* restore stack and value of x in input
		leas	7,s
		puls	x
		tstb
		lbne	ERRCODE
		rts

****************************************************
* sub WRITE sector of 256 Bytes
* Input A=Track, B= Sector, X = Buffer Address
* Output X preserved

WRITEFLX 	pshs b,x

* calculate X=LSN=SECNUM*T+S-1
		tfr x,y
		ldb	SECNUM,pcr
		mul
		tfr d,x
		puls b
		abx
		leax -1,x
		pshs	x	; stack = LSN/BufferAd

* stack bytes to send
		lda	DNUM,pcr
           clrb         
		pshs  a,b	
           lda   #$57
		pshs  a	; stack = $57/DNUM/0/LSN/BufferAd

* send  5 bytes to DW
        	leax  ,s  ; get start address
		ldy   #5
		lbsr  DWWrite
* write 256 bytes
		ldx   5,s	  ; get BufferAd
		ldy   #256			
		lbsr   DWWrite 

* calculate x = checksum and send to DW
		ldy   5,s	  ; get BufferAd
		ldx	 #0
		clra
LOOP2 	ldb	 ,y+
		abx
		inca
		bne LOOP2	; count A = 256 bytes
		pshs	x
		leax	,s	; get start address
		ldy   #2
		lbsr  DWWrite
* get control code
		leax  ,s    ; get start address
		ldy   #1
		ldd   #DELAY
		lbsr   DWRead
		lbne	NOTREADY
		ldb   ,s
		stb	WriteErr
* restore stack and value of x in input
		leas	7,s
		puls	x
		cmpb	#$F2
		beq	WProtect
		tstb
		lbne	ERRCODE
		rts
WProtect	ldb	#$40		; disk is Write Protected
		rts


*******************************************************
*        DWRead & DWWrite of HDBDOS V1.5 
* 57600 (115200) bps using 6809 code and timimg
* for real Dragon, changed if emulation

 ORG $51A0
 PUT *

**********************************************************
DWRead    clra          ; clear Carry (no framing error)
          deca          ; clear Z flag, A = timeout msb ($ff)
          tfr       cc,b
          pshs      u,x,dp,b,a     ; preserve registers, push timeout msb
          orcc      #IntMasks      ; mask interrupts
          tfr       a,dp           ; set direct page to $FFxx
          setdp     $ff
          leau      ,x             ; U = storage ptr
          ldx       #0             ; initialize checksum
          lda       #$01           ; A = serial in mask
          bra       rx0030         ; go wait for start bit

* Read a byte
rxByte    leau      1,u            ; bump storage ptr
          leay      ,-y            ; decrement request count
          lda       <BBIN          ; read bit 0
          lsra                     ; move bit 0 into Carry
          ldd       #$ff20         ; A = timeout msb, B = shift counter
          sta       ,s       ; reset timeout msb for next byte
          rorb           ; rotate bit 0 into byte accumulator
rx0010    lda       <BBIN      ; read bit (d1, d3, d5)
          lsra
          rorb
          bita      1,s        ; 5 cycle delay
          bcs       rx0020     ; exit loop after reading bit 5
          lda       <BBIN      ; read bit (d2, d4)
          lsra
          rorb
          leau      ,u
          bra       rx0010

rx0020    lda       <BBIN      ; read bit 6
          lsra
          rorb
          leay      ,y         ; test request count
          beq       rx0050  ; branch if final byte of request
          lda       <BBIN      ; read bit 7
          lsra
          rorb                 ; byte is now complete
          stb       -1,u       ; store received byte to memory
          abx                           ; update checksum
          lda       <BBIN               ; read stop bit
          anda      #$01                ; mask out other bits
          beq       rxExit         ; exit if framing error

* Wait for a start bit or timeout
rx0030    bita      <BBIN               ; check for start bit
          beq       rxByte     ; branch if start bit detected
          bita      <BBIN               ; again
          beq       rxByte
          ldb       #$ff                ; init timeout lsb
rx0040    bita      <BBIN
          beq       rxByte
          subb      #1               ; decrement timeout lsb
          bita      <BBIN
          beq       rxByte
          bcc       rx0040		; loop until timeout lsb rolls under
          bita      <BBIN
          beq       rxByte
          addb      ,s                  ; B = timeout msb - 1
          bita      <BBIN
          beq       rxByte
          stb       ,s         ; store decremented timeout msb
          bita      <BBIN
          beq       rxByte
          bcs       rx0030    ; loop if timeout hasn't expired
          bra       rxExit     ; exit due to timeout

rx0050    lda       <BBIN      ; read bit 7 of final byte
          lsra
          rorb                 ; byte is now complete
          stb       -1,u       ; store received byte to memory
          abx                  ; calculate final checksum
          lda       <BBIN      ; read stop bit
          anda      #$01       ; mask out other bits
          ora       #$02  ; return SUCCESS if no framing error

* Clean up, set status and return
rxExit    leas      1,s     ; remove timeout msb from stack
          inca         ; A = status to be returned in C and Z
          ora       ,s   ; place status information into the..
          sta       ,s    ; ..C and Z bits of the preserved CC
          leay      ,x                  ; return checksum in Y
          puls      cc,dp,x,u,pc	; restore registers and return
          setdp     $00

************************************************
DWWrite   pshs      dp,d,cc        ; preserve registers
          orcc      #IntMasks      ; mask interrupts
          ldd       #$04ff         ; A = loop counter, B = $ff
          tfr       b,dp           ; set direct page to $FFxx
          setdp     $ff
          ldb       <$ff23     ; read PIA 1-B control register
          andb      #$f7           ; clear sound enable bit
          stb       <$ff23         ; disable sound output
          fcb       $8c            ; skip next instruction
txByte    stb       <BBOUT         ; send stop bit
          ldb       ,x+            ; get a byte to transmit
          nop
          lslb         ; left rotate the byte two positions..
          rolb         ; ..placing a zero (start bit) in bit 1
tx0020    stb       <BBOUT 	; send bit (start bit, d1, d3, d5)
          rorb                ; move next bit into position
          exg       a,a
          nop
          stb       <BBOUT        ; send bit (d0, d2, d4, d6)
          rorb                  ; move next bit into position
          leau      ,u
          deca                    ; decrement loop counter
          bne       tx0020        ; loop until 7th data bit has been sent
          stb       <BBOUT        ; send bit 7
          ldd       #$0402  ; A = loop counter, B = MARK value
          leay      ,-y          ; decrement byte counter
          bne       txByte       ; loop if more to send
          stb       <BBOUT       ; leave bit banger output at MARK
          puls      cc,d,dp,pc  ; restore registers and return
          setdp     $00

